
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "assert.h"
#include "types.h"
#include "alpha.h"
#include "display.h"

#define ABS(x) ((x) >= 0? (x): -(x))

static possible_move *convert_to_move(const char *text, const board *b);


static void
display_board_only(const board *b, int indent)
{
    int i, j;
    printf("%-*s  ", indent, "");
    for (i=0; i < b->w; ++i)
      printf("%c", 'a'+i);
    printf("\n\n");
    for (j=0; j < b->h; ++j) {
        printf("%-*s%c ", indent, "", '1'+j);
        for (i=0; i < b->w; ++i) {
            if (is_present(b, i, j)) printf("O");
            else printf(".");
        }
        printf("\n");
    }
}


void display_board(const char *msg, const board *b)
{
    printf("    %s\n\n", msg);
    display_board_only(b, 4);
}

possible_move *get_users_move(const board *b)
{
    char buffer[100];
    possible_move *m;

    puts("Enter your move, e.g., 1a; D5.");

    while (1) {
        assert(fgets(buffer, sizeof buffer, stdin) != NULL);
        m = convert_to_move(buffer, b);

        if (m == NULL) {
            puts("That move didn't parse or wasn't valid; try again.");
            continue;
        }
        else break;  /* accept without confirmation */
    }

    return m;
}


static possible_move *
convert_to_move(const char *text, const board *b)
{
    int i = 0;
    possible_move *m = malloc(sizeof *m);
    m->x = m->y = -1;

    while (1) {
        while (text[i] && isspace(text[i])) ++i;
        if (text[i] == '\0') break;
        else if (isalpha(text[i]) && m->x < 0) {
            m->x = toupper(text[i])-'A';
            ++i;
        }
        else if (isdigit(text[i]) && m->y < 0) {
            if (m->y == '0') goto failed;
            m->y = text[i]-'1';
            ++i;
        }
        else goto failed;
    }

    /* Parsed the whole line. */
    if (m->x < 0 || m->y < 0) goto failed;
    if (!is_present(b, m->x, m->y)) goto failed;

    /* Found a complete move. */
    return m;

  failed:
    free(m);
    return NULL;
}
