
#include <stdlib.h>
#include <string.h>
#include "assert.h"
#include "types.h"


board *new_board(int w, int h)
{
    board *b = malloc(sizeof *b);
    assert(b);
    b->turn = PLAYER1;
    b->w = w; b->h = h;
    assert(w > 0 && h > 0);
    b->data = malloc(w*h * sizeof *b->data);
    assert(b->data);
    memset(b->data, 0x01, w*h);
    b->npresent = w*h;
    return b;
}

board *copy_board(const board *b)
{
    board *b2 = malloc(sizeof *b2);
    assert(b2);
    memcpy(b2, b, sizeof *b);
    b2->data = malloc(b->w * b->h * sizeof *b->data);
    assert(b2->data);
    memcpy(b2->data, b->data, b->w * b->h * sizeof *b->data);
    return b2;
}

void kill_board(board *b)
{
    free(b->data);
    free(b);
}

int is_present(const board *b, int x, int y)
{
    assert(0 <= x && x < b->w);
    assert(0 <= y && y < b->h);
    return (b->data[y*b->w + x] > 0);
}

void knock_out(board *b, int x, int y)
{
    int i, j;
    assert(is_present(b, x, y));
    b->data[y*b->w + x] = 0;
  repeat_until_done:
    for (j=0; j < b->h; ++j) {
        for (i=0; i < b->w; ++i) {
            int c, r;
            if (b->data[j*b->w + i] != 1) continue;
            /* Check whether this one is supported. */
            c = i;
            for (r = j+1; r < b->h; ++r) {
                if (!is_present(b, c, r)) goto try_horiz;
                if (b->data[r*b->w + c] == 2) break;
            }
            for (r = j-1; r >= 0; --r) {
                if (!is_present(b, c, r)) goto try_horiz;
                if (b->data[r*b->w + c] == 2) break;
            }
            /* It's supported by a vertical semi-arch. */
            b->data[j*b->w + i] = 2;
            goto repeat_until_done;

          try_horiz:
            r = j;
            for (c = i+1; c < b->w; ++c) {
                if (!is_present(b, c, r)) goto failed;
                if (b->data[r*b->w + c] == 2) break;
            }
            for (c = i-1; c >= 0; --c) {
                if (!is_present(b, c, r)) goto failed;
                if (b->data[r*b->w + c] == 2) break;
            }
            /* It's supported by a horizontal semi-arch. */
            b->data[j*b->w + i] = 2;
            goto repeat_until_done;

          failed:
            /* It's not supported; pass over it. */
            continue;
        }
    }

    /*
       We've marked all the supported blocks.
       Now the |data| array looks something like this:

         00220
         22222   <-- all supported blocks are marked;
         22201       falling blocks still show "1"
         00200
         00200
    */
    b->npresent = 0;
    for (i=0; i < b->w*b->h; ++i) {
        b->data[i] = (b->data[i] == 2);
        b->npresent += b->data[i];
    }

    b->turn = (b->turn == PLAYER2)? PLAYER1: PLAYER2;
    return;
}
